module EasySitemap
  class Runner

    attr_reader :result

    def initialize(urls, api_key: nil)
      @urls = urls
      @api_key = api_key
      @result = {}
    end

    def self.perform(filename: 'sitemap.xml', thread_count: 5, api_key: nil)
      doc = File.open(filename) { |f| Nokogiri::XML(f) }

      runners = doc.xpath("//loc").
          map(&:text).
          in_groups(thread_count, false).
          map { |group| Runner.new(group, api_key: api_key) }

      threads = runners.map { |runner| Thread.new { runner.perform } }
      threads.each { |thr| thr.join }

      runners.map(&:result).inject(:merge)
    end

    def perform
      @urls.each do |url|
        ping_url(url)
      end
    end

    private

    def ping_url(url)
      r = {}

      starting = Process.clock_gettime(Process::CLOCK_MONOTONIC)

      begin
        h = {}
        h['X-Redmine-API-Key'] = @api_key if @api_key.present?

        response = HTTParty.get(url, follow_redirects: false, headers: h)

        r['status'] = response.code
      rescue Errno::ECONNREFUSED
        r['status'] = 410
      end

      ending = Process.clock_gettime(Process::CLOCK_MONOTONIC)
      r['time'] = (ending - starting).round(2)

      @result[url] = r
    end

  end
end
